body {
    background-color: lavender; /* Sets a solid background color */
}

dt {
    font-weight: bold;
}
span.nature {
    font-style: italic;
}
span.required {
    font-style: italic;
    color: red;
}

span.optional {
    font-style: italic;
    color: teal;
}
